#!/usr/bin/env bash
#set -e
#set -u
#set -o pipefail

SCRIPT=$(readlink -f $0)
SCRIPT_PATH=$(dirname $SCRIPT)
AQUAMIS_PATH=$(dirname $SCRIPT_PATH)

# Init Conda and Install Mamba
CONDA_BASE=$(conda info --base)
source $CONDA_BASE/etc/profile.d/conda.sh
conda install --channel conda-forge mamba

# Create Conda Environment for AQUAMIS
CONDA_ENV_NAME=$(head -1 ${AQUAMIS_PATH}/envs/aquamis.yaml | cut -d' ' -f2)
mamba env create -f ${AQUAMIS_PATH}/envs/aquamis.yaml
conda activate $CONDA_ENV_NAME

# Create Subdirectories
mkdir -p ${AQUAMIS_PATH}/reference_db/mash
mkdir -p ${AQUAMIS_PATH}/reference_db/taxonkit
mkdir -p ${AQUAMIS_PATH}/test_data
mkdir -p ${AQUAMIS_PATH}/download && cd ${AQUAMIS_PATH}/download

# Download Databases and Test Data
wget --output-document augustus.tar.gz     https://seafile.bfr.berlin/f/64cc5034fad74f50a2f0/?dl=1  # 139MB or https://databay.bfrlab.de/f/fa13c9eb2625477eb729/?dl=1
wget --output-document bacteria.tar.gz     https://seafile.bfr.berlin/f/41cf8fdcfe2043d2800e/?dl=1  # 8.8MB or https://busco.ezlab.org/v2/datasets/bacteria_odb9.tar.gz
wget --output-document confindr_db.tar.gz  https://seafile.bfr.berlin/f/ede87ec860624a0cb406/?dl=1  # 173MB
wget --output-document mashDB.tar.gz       https://seafile.bfr.berlin/f/41f804a1eba541788530/?dl=1  # 216MB or https://databay.bfrlab.de/f/34b8c88945a8439dac64/?dl=1
wget --output-document minikraken2.tar.gz  https://seafile.bfr.berlin/f/8ca1b4d2c97341498698/?dl=1  # 6.0GB or ftp://ftp.ccb.jhu.edu/pub/data/kraken2_dbs/minikraken_8GB_202003.tgz
wget --output-document taxdump.tar.gz      https://seafile.bfr.berlin/f/1d51700ecfd241e4a6d4/?dl=1  #  54MB or ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz
wget --output-document raw.tar.gz          https://seafile.bfr.berlin/f/005aa1249bb14193be3a/?dl=1  # 1.8GB or https://databay.bfrlab.de/f/bf82c166df1a41d59a61/?dl=1

# Verify Integrity of Downloads
openssl dgst -r -sha256 * > download.sha256

# Unpack Downloads
tar -xzv -f augustus.tar.gz    -C ${CONDA_BASE}/envs/${CONDA_ENV_NAME}/lib/python3.7/site-packages/quast_libs/augustus3.2.3/
tar -xzv -f bacteria.tar.gz    -C ${CONDA_BASE}/envs/${CONDA_ENV_NAME}/lib/python3.7/site-packages/quast_libs/busco/
tar -xzv -f confindr_db.tar.gz -C ${AQUAMIS_PATH}/reference_db/
tar -xzv -f mashDB.tar.gz      -C ${AQUAMIS_PATH}/reference_db/mash/
tar -xzv -f minikraken2.tar.gz -C ${AQUAMIS_PATH}/reference_db/
tar -xzv -f taxdump.tar.gz     -C ${AQUAMIS_PATH}/reference_db/taxonkit/
tar -xzv -f raw.tar.gz         -C ${AQUAMIS_PATH}/test_data/

# CleanUp
mv ${AQUAMIS_PATH}/download/download.sha256 ${AQUAMIS_PATH}/reference_db/download.sha256
#rm -R ${AQUAMIS_PATH}/download

# Create Sample Sheet for Test Data
bash ${SCRIPT_PATH}/create_sampleSheet.sh --mode illumina --fastxDir ${AQUAMIS_PATH}/test_data/raw --outDir ${AQUAMIS_PATH}/test_data
